<?php
/*
Plugin Name: iTunes Lookup Widget
Description: Embeds an iTunes Widget to display any kind of store items (software, albums, artists etc).
Author: ollybach
Version: 0.1
License:

  Copyright 2012 iTunes Lookup Widget (dev@1000db.com)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2, as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

register_activation_hook( __FILE__, 'ilw_check_plugin_requirements');
register_uninstall_hook( __FILE__, 'ilw_uninstall' );

//PHP 5.2 (json_decode) as well as cUrl is required , so if PHP version is lower then 5.2, display an error message and deactivate the plugin
function ilw_check_plugin_requirements(){
	if( version_compare( PHP_VERSION, '5.2', '<' ) || !in_array('curl', get_loaded_extensions()) ) {
		if( is_admin()) {
			require_once ABSPATH . '/wp-admin/includes/plugin.php';
			deactivate_plugins( basename( __FILE__ ) );
			wp_die( 'iTunes Lookup Widget requires the server on which your site resides to be running PHP 5.2 or higher as well as cUrl. As of version 3.2, WordPress itself will also <a href="http://wordpress.org/news/2010/07/eol-for-php4-and-mysql4">have this requirement</a>. You should get in touch with your web hosting provider and ask them to update PHP.<br /><br /><a href="' . admin_url( 'plugins.php' ) . '">Back to Plugins</a>' );
		}
	}
}


class iTunes_Lookup_Widget extends WP_Widget {
	private $pluginVersion;
	private $pluginName;
	private $pluginSlug;
	private $pluginLocale;
	private $pluginTitle;
	private $pluginLocalizedVariables;
	private $pluginVariables;
	private $pluginVariablesName;
	private $pluginOptionsName;
	private $pluginItunesVersion;
	private $pluginItunesLimit;


	private $pluginJsInFooter;
	private $pluginCacheExpiry;
	private $pluginCacheCleanUp;


    /*--------------------------------------------------*/
    /* Constructor
    /*--------------------------------------------------*/


    /**
     * The widget constructor. Specifies the classname and description, instantiates
     * the widget, loads localization files, and includes necessary scripts and
     * styles.
     */
     function __construct() {

		/**init constants***/
		$this->pluginVersion='0.1';
	 	$this->pluginName="iTunes Lookup Widget";
	 	$this->pluginSlug="itunes-lookup-widget";
	 	$this->pluginLocale="ilw-locale";
	 	$this->pluginTitle="on iTunes";
	 	$this->pluginLocalizedVariables="ilwParam";
	 	$this->pluginVariables=array();
	 	$this->pluginSettingsName='settings';
	 	$this->pluginInstanceName='instances';
	 	$this->pluginRequestName='request';
	 	$this->pluginVariablesName='vars';
	 	$this->pluginOptionsName='ilw_options';//set also in uninstall
	 	$this->pluginItunesVersion='2';
	 	$this->pluginItunesLimit='200';/*max itunes will return, defaults to 50 if not set*/



		$options = get_option($this->pluginOptionsName);

	 	$this->pluginJsInFooter=$options['js_in_footer'];
	 	$this->pluginCacheExpiry=$options['cache_expiry'];
	 	$this->pluginCacheCleanUp=$options['cache_cleanup'];

    	//  update classname and description
        $widget_opts = array (
            'classname' => $this->pluginSlug,
            'description' => __('Display items available on iTunes ', $this->pluginLocale)
        );

    	add_action('admin_menu', array( $this, 'plugin_admin_add_page' ) );
    	add_action('admin_init', array( $this, 'admin_options_init'));

		/**add settings to ajax call when getting iTunes json results, wp_ajax_[nopriv]_[slug] needs t0 be referenved in js files too***/
		add_action('wp_ajax_ilw_itunes_json', array(&$this,'ilw_itunes_json') );
		add_action('wp_ajax_nopriv_ilw_itunes_json', array(&$this,'ilw_itunes_json') ); // need this to serve non logged in users

        $this->WP_Widget(false, $name=$this->pluginName, $widget_opts);
        load_plugin_textdomain($this->pluginLocale, false, dirname(plugin_basename( __FILE__ ) ) . '/lang' );

    	// Load JavaScript and stylesheets
    	$this->register_scripts_and_styles();

    } // end constructor




    /*--------------------------------------------------*/
    /* API Functions
    /*--------------------------------------------------*/

	/************************************************************************************
     [add settings to ajax call and get iTunes json results]
    ************************************************************************************/
	public function ilw_itunes_json(){
		require(plugin_dir_path(__FILE__).'ajax/get-json.php');
		die();
	}
    /*******************************************************
     * Outputs the content of the widget.
     *
     * @args            The array of form elements
     * @instance
     ******************************************************/
    function widget($args, $instance) {
    	static $widgetcount=0;

		require(plugin_dir_path(__FILE__).'views/widget.php');
		/**localize variables in footer if set to footer***/
		if($this->pluginJsInFooter){
			$options = get_option($this->pluginOptionsName);
			/**instance variables**/
			/**instance query string only useful if an artistid is has been defined**/
			if($instance['artist_id']>0){
				$this->pluginVariables[''.$this->pluginInstanceName.''][''.$widget_id.'']=$this->number;
			}
		}
    	$widgetcount++;
    } // end widget


    /**
     * Processes the widget's options to be saved.
     *
     * @new_instance    The previous instance of values before the update.
     * @old_instance    The new instance of values to be generated via the update.
     */
    function update($new_instance, $old_instance) {

        $instance = $old_instance;

        $instance['title'] = strip_tags( stripslashes($new_instance['title']) );
        $instance['artist_id'] = $this->sanitize_selection( array($new_instance['artist_id']),'int',false );
        $instance['max_items'] = absint( $new_instance['max_items'] );
    	$instance['country'] = strip_tags( $new_instance['country'] );
    	$instance['language'] = strip_tags( $new_instance['language'] );
    	$instance['explicit'] = strip_tags( $new_instance['explicit'] );
    	$instance['exclude'] = empty($new_instance['exclude'] )? false : $this->sanitize_selection($new_instance['exclude']);
    	$instance['sort'] = $this->sanitize_selection($new_instance['sort'],'int' );
    	$instance['lbl'] = $this->sanitize_selection($new_instance['lbl']);
        $instance['show_label'] = empty($new_instance['show_label']) ? false : true;
        $instance['sortKey'] = strip_tags( $new_instance['sortKey'] );
        $instance['sortOrder'] = strip_tags( $new_instance['sortOrder'] );

        return $instance;

    } // end widget

    /*****************************************************
     * Generates the administration form for the widget.
     *
     * @instance    The array of keys and values for the widget.
     ******************************************************/
    function form($instance) {
        // Display the admin form
    	include(plugin_dir_path(__FILE__).'views/admin.php');
		/**add to localized admin vars if this is an active widget**/
		if ( is_active_widget( false, $this->id, $this->id_base, true ) ) {
			$this->pluginVariables[''.$this->pluginInstanceName.''][''.$this->id.'']=$this->number;
		}
    } // end form


    /*****************************************************
    *
    * 	[Admin Options Functions]
    *
    *****************************************************/
	/*Setup admin settings page*/
	function plugin_admin_add_page() {
		add_options_page('Settings', 'iTunes Lookup Widget', 'manage_options',  basename( __FILE__ ), array( $this, 'admin_page' ) );
	}
    /*display the admin options page*/
    function admin_page() {
		// Check if user can access to the plugin
		if (!current_user_can('administrator'))
		wp_die( __('You do not have sufficient permissions to access this page.') );
    ?>
		<div class="ilw-settings">
			<h2>iTunes Lookup Widget Options</h2>
			<?php settings_errors(); ?>

			<h3 class="nav-tab-wrapper">
			    <a href="#" class="nav-tab main_settings"><?php _e('Main Settings', $this->pluginLocale) ?></a>
			    <a href="#" class="nav-tab localize_settings"><?php _e('Localization', $this->pluginLocale)?></a>
			</h3>

			<form action="options.php" method="post">
			<?php settings_fields($this->pluginOptionsName); ?>

			<div id="main_settings" class="ilw-settings-tab"><?php do_settings_sections('settings_options'); ?></div>
			<div id="localize_settings" class="ilw-settings-tab"><?php do_settings_sections('localize_options'); ?></div>
			<?php submit_button( __('Save Changes', $this->pluginLocale) );  ?>

			</form>
		</div>

	<?php
	}

	function admin_options_init(){
		register_setting($this->pluginOptionsName,$this->pluginOptionsName, array( $this, 'options_validate') );

		/**main settings**/
		add_settings_section('plugin_main', __('Main Settings', $this->pluginLocale),  array( $this, 'main_section_text'), 'settings_options');

		add_settings_field('version', '<b>'.__('Plugin Version:', $this->pluginLocale).'</b>', array( $this, 'settings_input'), 'settings_options', 'plugin_main', 'version' );
		add_settings_field('cache_expiry', '<b>'.__('Cache Expiry Time:', $this->pluginLocale).'</b> '.__('[in Seconds] (0 to disable). I.e when do we re-fetch results from iTunes.<br/>Default 1 week.', $this->pluginLocale).'', array( $this, 'settings_input'), 'settings_options', 'plugin_main', 'cache_expiry' );
		add_settings_field('cache_cleanup', '<b>'.__('Cache Cleanup:', $this->pluginLocale).'</b> '.__('[in Seconds] If you change widget settings often (and have cache enabled) the cache will fill with all these requests. Set the time here how often you want to run the cleanup. It will only delete files that are older than cache settings above of course.<br/>Default 1 week', $this->pluginLocale).'', array( $this, 'settings_input'), 'settings_options', 'plugin_main', 'cache_cleanup' );
		add_settings_field('js_in_footer', '<b>'.__('Javascript in Footer:', $this->pluginLocale).'</b> '.__('[combines all jsVars in one tidy place, but requires wp_footer in theme]', $this->pluginLocale).'', array( $this, 'settings_input'), 'settings_options', 'plugin_main', 'js_in_footer' );
		add_settings_field('js_priority', '<b>'.__('Javascript Priority:', $this->pluginLocale).'</b> '.__('Some plugins change priority / loadorder to other than the default of 10. If you have problems try decreasing this number. For example , "javascript to footer" uses 5 in which case, use 4 here.  [only used when javascript in footer is enabled]', $this->pluginLocale).'', array( $this, 'settings_input'), 'settings_options', 'plugin_main', 'js_priority' );
		add_settings_field('include_css', '<b>'.__('Include CSS:', $this->pluginLocale).'</b> '.__('include frontend css that came with this plugin (untick if you want to provide your own styles somewhere else)', $this->pluginLocale).'', array( $this, 'settings_input'), 'settings_options', 'plugin_main', 'include_css' );
		add_settings_field('affiliates', '<b>'.__('Affiliate Settings:', $this->pluginLocale).'</b>'.__('<br/>please note: (as I understand it) affiliate links will only generate revenues in certain territories. If you have a tradedoubler account for - let\'s say - Italy[IT], you will have a program Id of 24373 . However, this ID is also valid to generate commission in and for the following countries and program id\'s<br/><b>[AT=>24380],[BE=>24379],[CH=>24372],[DE=>23761],[DK=>24375],[ES=>24364],[FI=>24366],[FR=>23753],[GB=>23708],[IE=>24367],[IT=>24373],[NL=>24371],[NO=>24369],[SE=>23762]</b><br/>(i.e. when you set your widget to search the stores in these countries)<br/><br/>Linktrade and DGM links will apply when your widget searches <b>[US,CA]</b> and <b>[AUS,NZ]</b> respectively (provided the affiliate id\'s are set).<br><br/><b>If you set your widget to search any other store than the ones mentioned above, the affiliate link will be ignored and a direct iTunes link will be used.</b> (this is also the case, if you have not set the relevant affiliate variables)<br/><br/>If you have any questions, or think I have gotten somthing wrong here, please let me know.<br/><br/><b>(please see the <a href="http://www.apple.com/itunes/affiliates/resources/documentation/linking-to-the-itunes-music-store.html#AddingAffiliateTracking">iTunes documentation</a> for details regarding affiliate links)</b>', $this->pluginLocale).'', array( $this, 'settings_input'), 'settings_options', 'plugin_main', 'affiliates' );


		/**localize frontend**/
		add_settings_section('plugin_localize', __('Localize Frontend', $this->pluginLocale),  array( $this, 'localize_section_text'), 'localize_options');
		foreach($this->ilw_localize_frontend('label') as $k=>$v){
			add_settings_field($k, ''.$v.'', array( $this, 'settings_input'), 'localize_options', 'plugin_localize', $k);
		}


		/**(re)-initialize setings*************/
		$this->initialize_option_settings();
	}

	function main_section_text() {
		echo '<h4>'.__('Set Options as required', $this->pluginLocale).'</h4>';
	}
	function localize_section_text() {
		echo '<h4>'.__('Use the fields below to localize the frontend labels. Leave empty if you do not want to display any particular label', $this->pluginLocale).'</h4>';
	}

	function settings_input($field='') {
		$options = get_option($this->pluginOptionsName);
		$localizeValues=unserialize($options['localized_frontend']);


		if($field=='version'){
			echo "{$options[$field]}";
		}
		if($field=='cache_expiry' || $field=='cache_cleanup' || $field=='js_priority'){
			echo "<input id='".$field."' name='".$this->pluginOptionsName."[".$field."]' size='20' type='text' value='{$options[$field]}' />";
		}
		if($field=='js_in_footer' || $field=='include_css' ){

			$docheck = $options[$field] ? 'checked="checked"' : '';
			echo "<input id='".$field."' name='".$this->pluginOptionsName."[".$field."]' type='checkbox'  ".$docheck." value='1' />";
		}

		if(in_array($field,array_keys($this->ilw_localize_frontend()))){
			echo "<input id='".$field."' name='".$this->pluginOptionsName."[localized_frontend][".$field."]' size='20' type='text' value='".$localizeValues[$field]."' />";
		}

		if($field=='affiliates'){

			$items=$this->ilw_affiliate_options();
			$affVal=unserialize($options['affiliate_values']);

			/***create output***/
			$affiliateOptions='';
			$affiliateOptions.='<div class="ilw-aff-optn">';
			foreach($items as $k=>$v){
				if($options['affiliate_type']==''.$k.''){
					$docheck = 'checked="checked"';
				}else{$docheck = '';}
				$affiliateOptions.='<input type="radio" name="'.$this->pluginOptionsName.'[affiliate_type]" value="'.$k.'" '.$docheck.'>';
				$affiliateOptions.=' '.$v['txt'].'<br/>';
				if(isset($v['territories'])){
					if($v['inputtype']!='hidden'){
						$affiliateOptions.='<b>Territories:</b><br/>';
					}
					foreach($v['territories'] as $tk=>$tv){
						if($v['inputtype']!='hidden'){
							$affiliateOptions.='<div class="ilw-aff-optn-inp" ><span>'.$v['territories'][$tk]['txt'].':</span>';
						}
						if(isset($v['territories'][$tk]['programVars'])){
						foreach($v['territories'][$tk]['programVars'] as $pk=>$pv){
							if($v['inputtype']!='hidden'){
								$affiliateOptions.='<label>';
								$affiliateOptions.=''.$pv['txt'].': ';
							}
							$affiliateOptions.='<input type="'.$v['inputtype'].'" name="'.$this->pluginOptionsName.'[affiliate_values]['.$k.']['.$tk.']['.$pv['key'].']" value="'.htmlspecialchars($affVal[$k][$tk][$pv['key']]).'"/>';//'.$affVal[$k][$tk][$pv['key']].'
							if($v['inputtype']!='hidden'){
								$affiliateOptions.='</label>';
							}
						}}
						if($v['inputtype']!='hidden'){
						$affiliateOptions.='</div>';
						}
					}
				}
				$affiliateOptions.='<br/>';
			}
			/**add donate button**/
			$affiliateOptions.='<br/>'.__('If you would like to contribute directly to the development of this plugin, feel free to do so via paypal below.', $this->pluginLocale).'<br/>'.__('Thank you, much appreciated.', $this->pluginLocale).'<br/>';
			$affiliateOptions.='<form action="https://www.paypal.com/cgi-bin/webscr" method="post">
			<input type="hidden" name="cmd" value="_s-xclick">
			<input type="hidden" name="hosted_button_id" value="BEUAXQHT7FQ3Y">
			<input type="image" src="https://www.paypalobjects.com/en_US/GB/i/btn/btn_donateCC_LG.gif" border="0" name="submit" alt="PayPal — The safer, easier way to pay online.">
			<img alt="" border="0" src="https://www.paypalobjects.com/en_GB/i/scr/pixel.gif" width="1" height="1">
			</form>';
			$affiliateOptions.='</div>';
			/**print above**/
			print"".$affiliateOptions."";
		}
	}


	function initialize_option_settings(){
		// Check for capability
		if ( !current_user_can('activate_plugins') ){
			return;
		}
		$options = get_option($this->pluginOptionsName);

			$defaults = array(
				'version' => $this->pluginVersion,
				'cache_expiry' => (86400*7),
				'cache_cleanup' => (86400*7),
				'js_in_footer' => true,
				'js_priority' => 10,
				'include_css' => true,
				'affiliate_type' => 'default',
				'affiliate_values' => $this->ilw_affiliate_options(array('default','custom')),
				'affiliate_selected' => $this->ilw_affiliate_options('default'),
				'localized_frontend' => $this->ilw_localize_frontend()
			);

			foreach($defaults as $key=>$val){
				if(!isset($options[$key])){
					add_option($this->pluginOptionsName, $key);//must also be in options_validate
					$newVal[$key]=$val;
				}
			}
			if(isset($newVal)){
				update_option($this->pluginOptionsName, $newVal );
			}
	}

    function options_validate($input){
		$options = get_option($this->pluginOptionsName);

		if(isset($input['version'])){
			$options['version'] = trim(preg_replace("/[^0-9\.]/","", $input['version']));
		}
		if(isset($input['cache_expiry'])){
			$options['cache_expiry'] = trim(preg_replace("/[^0-9]/","", $input['cache_expiry']));
		}
		if(isset($input['cache_cleanup'])){
			$options['cache_cleanup'] = trim(preg_replace("/[^0-9]/","", $input['cache_cleanup']));
		}

			$options['js_in_footer'] = isset($input['js_in_footer']) ? true : false;

		if(isset($input['js_priority'])){
			$options['js_priority'] = (preg_replace("/[^0-9]/","", $input['js_priority']));
		}

			$options['include_css'] = isset($input['include_css']) ? true : false;

		/*********insert default affiliate values , if updated from settings page, override *****************************/
		if(isset($input['affiliate_type']) || isset($input['affiliate_values']) || isset($input['affiliate_selected'])){
			/*insert default value or update **/
			if(isset($input['affiliate_type'])){
				$options['affiliate_type'] = (preg_replace("/[^a-zA-Z_]/","", $input['affiliate_type']));
			}
			/*insert default value, if updated this gets overwritten below**/
			if(isset($input['affiliate_values'])){
				$options['affiliate_values'] = serialize($input['affiliate_values']);
			}
			/*insert default value, if updated this gets overwritten below**/
			if(isset($input['affiliate_selected'])){
				$options['affiliate_selected'] = json_encode($input['affiliate_selected']);
			}

			/***if we are actually updating the settings , override affiliate_type and affiliate_selected here**/
			if(isset($input['affiliate_type']) && isset($input['affiliate_values']) && is_array($input['affiliate_values'])){
			$thisOption=array();
			$selectedOption=array();
				foreach($input['affiliate_values'] as $k=>$v){
					foreach($v as $l=>$m){
						$count=0;
						foreach($m as $x=>$y){
							/**Tradedoubler and DGM are using integers, Linkshare ids (and others) might have all sorts of characters (and i have no idea which ones, hence just a  strip tags)**/
							if($l=='EU' || $l=='AUSNZ' ){
								$thisOption[$k][$l][$x]=preg_replace("/[^0-9]/","", $y);
							}else{
								$thisOption[$k][$l][$x]=strip_tags($y);
							}
							/**check to see if any field associated with each territory is empty**/
							if($k==$options['affiliate_type'] && $thisOption[$k][$l][$x]!=''){
								$usedOption[$l][$x]=$thisOption[$k][$l][$x];
							}
						$count++;
						}
						/**do not save selected/used values when one or more variables/fields are empty of any given territory**/
						if(isset($usedOption[$l]) && count($usedOption[$l])==$count){
							/**set to be used option to have the same key**/
							$selectedOption['affiliates'][$l]=$usedOption[$l];
						}
					}
				}

				$options['affiliate_values'] = serialize($thisOption);

				if($options['affiliate_type']=='custom' || $options['affiliate_type']=='default'){
					if(count($selectedOption)<=0){$selectedOption=new StdClass();}
					$options['affiliate_selected'] = json_encode($selectedOption);
				}else{
					$options['affiliate_selected']=json_encode($options['affiliate_type']);
				}
			}
		}

		if(isset($input['localized_frontend'])){
			$options['localized_frontend'] = $this->ilw_serialize_option_array($input['localized_frontend']);
		}

		return $options;
    }

    /*--------------------------------------------------*/
    /* Private Functions
    /*--------------------------------------------------*/
   /****************************************************
   	[sanitize some input]
   ****************************************************/

	private function sanitize_selection($str,$type='str',$multiple=true) {

		$val=array();
		if(is_array($str)){
		foreach($str as $value){
			if($type=='int'){
				$item=trim(preg_replace("/[^0-9]/","", $value));
			}
			if($type=='str'){
				/**replace komma with entity so we can safely implode below**/
				$item=str_replace(",","&#44;",$value);
				$item=''.sanitize_text_field($item).'';
			}

				$val[]=$item;
		}
		}
		/**if we want to enable more than one as comma seperated array**/
		if($multiple){
			return implode(",",$val);
		}else{
			return $val[0];
		}
	}
	/*******************************************************************************
		[serialize an array of multiple fields into one option value keeping keys]
	*******************************************************************************/
	private function ilw_serialize_option_array($array) {
		$val=array();
		if(is_array($array)){
		foreach($array as $key=>$value){
			$val[$key]=htmlentities($value,ENT_QUOTES);//strip_tags
		}}
		return serialize($val);
	}

    /**
     * Registers and enqueues stylesheets for the administration panel and the
     * public facing site.
     */
    private function register_scripts_and_styles() {
      $options = get_option($this->pluginOptionsName);
        if(is_admin()) {
            /**css**/
				wp_register_style($this->pluginSlug, plugins_url( 'css/styles-admin.css', __FILE__ ));
 				wp_enqueue_style($this->pluginSlug);
      		/**js***/
            	wp_register_script($this->pluginSlug, plugins_url( 'js/scripts-admin.js', __FILE__ ), array('jquery'), null ,true);
            	wp_enqueue_script($this->pluginSlug);

			/**localize search variables**/
			$this->pluginVariables=array('url'=>admin_url( 'admin-ajax.php'));
			add_action('admin_footer',  array(&$this, 'localize_variables'));

        } else {
        	/**only include if there's an active widget**/
        	if ( is_active_widget( false, false, $this->id_base, true ) ) {
            	/**css**/
            	if($options['include_css']){
					wp_register_style($this->pluginSlug, plugins_url( 'css/styles.css', __FILE__ ));
 					wp_enqueue_style($this->pluginSlug);
            	}
        		/**js***/
            		wp_register_script($this->pluginSlug, plugins_url( 'js/scripts.min.js', __FILE__ ), array('jquery'), null ,$this->pluginJsInFooter);
            		wp_enqueue_script($this->pluginSlug);

				/**localize variables******************************/
				$this->pluginVariables=array('url'=>admin_url( 'admin-ajax.php'));

				/**localize in footer if set otherwise these variables will be set under each widget element***/
				if($this->pluginJsInFooter){
					add_action('wp_footer',  array(&$this, 'localize_variables'),$options['js_priority']);
				}
			}
        } // end if/else
    } // end register_scripts_and_styles


    /**
     * Helper function for  localizing scripts.
     *
     * @dependency    	The  that has been registered with WordPress
     * @file_path       the js variable to use
     * @parameters    the parameters to output
     */
	 public function localize_variables(){
//		if(is_admin()) {
			wp_localize_script( $this->pluginSlug, $this->pluginLocalizedVariables,  $this->pluginVariables);
//		}else{
//			wp_localize_script( $this->pluginSlug, $this->pluginLocalizedVariables,  $this->pluginVariables);
//		}
	 }
    /*********************************************************
      [Helper function to find artist id in admin widget]
    *********************************************************/
//    private function ilw_admin_search_options($key1=false,$key2=false) {
		//			'version'=>$this->pluginItunesVersion,
		//$items[''.$this->pluginSettingsName.'']=array(
		//	'url' =>  admin_url( 'admin-ajax.php' ),
		//);


//		$items['url']=admin_url( 'admin-ajax.php');


//			'type'=>'search',
//			'limit'=>$this->pluginItunesLimit


//		$items['ajaxurl']=array(
//			'url'=>admin_url('admin-ajax.php')
//		);
//		$items['searchtype']=array(
//			'type'=>'search'
//		);
//		$items['request']=array(
//			'version'=>$this->pluginItunesVersion,
//			'media'=>'all',
//			'entity'=>'allArtist',
//			'attribute'=>'allArtistTerm',
//			'limit'=>$this->pluginItunesLimit
//			);
//
//		if($key1){$items=$items[$key1];}
//		if($key1 && $key2){$items=$items[$key2];}

//		return $items;
//    }

    /*********************************************************
      [js object vars on frontend]
    *********************************************************/
//    private function ilw_public_js_options() {
//    	//'version'=>$this->pluginItunesVersion,
//		$items[''.$this->pluginSettingsName.'']=array(
//			'url' =>  admin_url( 'admin-ajax.php' ),
//			'type'=>'lookup',
//			'limit'=>$this->pluginItunesLimit
//		);
//		
//		$items['url']=admin_url( 'admin-ajax.php');
//		
//		
//		return $items;
//    }

	/*********************************************************
		[country list to determine which store to search]
	*********************************************************/
	private function ilw_country_list($selected=''){
	    $items = array(
			array('iso'=>'GB','name'=>'United Kingdom'),
			array('iso'=>'US','name'=>'United States'),
			array('iso'=>'AF','name'=>'Afghanistan'),
			array('iso'=>'AL','name'=>'Albania'),
			array('iso'=>'DZ','name'=>'Algeria'),
			array('iso'=>'AS','name'=>'American Samoa'),
			array('iso'=>'AD','name'=>'Andorra'),
			array('iso'=>'AO','name'=>'Angola'),
			array('iso'=>'AI','name'=>'Anguilla'),
			array('iso'=>'AQ','name'=>'Antarctica'),
			array('iso'=>'AG','name'=>'Antigua And Barbuda'),
			array('iso'=>'AR','name'=>'Argentina'),
			array('iso'=>'AM','name'=>'Armenia'),
			array('iso'=>'AW','name'=>'Aruba'),
			array('iso'=>'AU','name'=>'Australia'),
			array('iso'=>'AT','name'=>'Austria'),
			array('iso'=>'AZ','name'=>'Azerbaijan'),
			array('iso'=>'BS','name'=>'Bahamas'),
			array('iso'=>'BH','name'=>'Bahrain'),
			array('iso'=>'BD','name'=>'Bangladesh'),
			array('iso'=>'BB','name'=>'Barbados'),
			array('iso'=>'BY','name'=>'Belarus'),
			array('iso'=>'BE','name'=>'Belgium'),
			array('iso'=>'BZ','name'=>'Belize'),
			array('iso'=>'BJ','name'=>'Benin'),
			array('iso'=>'BM','name'=>'Bermuda'),
			array('iso'=>'BT','name'=>'Bhutan'),
			array('iso'=>'BO','name'=>'Bolivia'),
			array('iso'=>'BA','name'=>'Bosnia And Herzegowina'),
			array('iso'=>'BW','name'=>'Botswana'),
			array('iso'=>'BV','name'=>'Bouvet Island'),
			array('iso'=>'BR','name'=>'Brazil'),
			array('iso'=>'IO','name'=>'British Indian Ocean Territory'),
			array('iso'=>'BN','name'=>'Brunei Darussalam'),
			array('iso'=>'BG','name'=>'Bulgaria'),
			array('iso'=>'BF','name'=>'Burkina Faso'),
			array('iso'=>'BI','name'=>'Burundi'),
			array('iso'=>'KH','name'=>'Cambodia'),
			array('iso'=>'CM','name'=>'Cameroon'),
			array('iso'=>'CA','name'=>'Canada'),
			array('iso'=>'CV','name'=>'Cape Verde'),
			array('iso'=>'KY','name'=>'Cayman Islands'),
			array('iso'=>'CF','name'=>'Central African Republic'),
			array('iso'=>'TD','name'=>'Chad'),
			array('iso'=>'CL','name'=>'Chile'),
			array('iso'=>'CN','name'=>'China'),
			array('iso'=>'CX','name'=>'Christmas Island'),
			array('iso'=>'CC','name'=>'Cocos (Keeling) Islands'),
			array('iso'=>'CO','name'=>'Colombia'),
			array('iso'=>'KM','name'=>'Comoros'),
			array('iso'=>'CG','name'=>'Congo'),
			array('iso'=>'CD','name'=>'Congo, The Democratic Republic Of The'),
			array('iso'=>'CK','name'=>'Cook Islands'),
			array('iso'=>'CR','name'=>'Costa Rica'),
			array('iso'=>'CI','name'=>'Cote D\'Ivoire'),
			array('iso'=>'HR','name'=>'Croatia (Local Name: Hrvatska)'),
			array('iso'=>'CU','name'=>'Cuba'),
			array('iso'=>'CY','name'=>'Cyprus'),
			array('iso'=>'CZ','name'=>'Czech Republic'),
			array('iso'=>'DK','name'=>'Denmark'),
			array('iso'=>'DJ','name'=>'Djibouti'),
			array('iso'=>'DM','name'=>'Dominica'),
			array('iso'=>'DO','name'=>'Dominican Republic'),
			array('iso'=>'TP','name'=>'East Timor'),
			array('iso'=>'EC','name'=>'Ecuador'),
			array('iso'=>'EG','name'=>'Egypt'),
			array('iso'=>'SV','name'=>'El Salvador'),
			array('iso'=>'GQ','name'=>'Equatorial Guinea'),
			array('iso'=>'ER','name'=>'Eritrea'),
			array('iso'=>'EE','name'=>'Estonia'),
			array('iso'=>'ET','name'=>'Ethiopia'),
			array('iso'=>'FK','name'=>'Falkland Islands (Malvinas)'),
			array('iso'=>'FO','name'=>'Faroe Islands'),
			array('iso'=>'FJ','name'=>'Fiji'),
			array('iso'=>'FI','name'=>'Finland'),
			array('iso'=>'FR','name'=>'France'),
			array('iso'=>'FX','name'=>'France, Metropolitan'),
			array('iso'=>'GF','name'=>'French Guiana'),
			array('iso'=>'PF','name'=>'French Polynesia'),
			array('iso'=>'TF','name'=>'French Southern Territories'),
			array('iso'=>'GA','name'=>'Gabon'),
			array('iso'=>'GM','name'=>'Gambia'),
			array('iso'=>'GE','name'=>'Georgia'),
			array('iso'=>'DE','name'=>'Germany'),
			array('iso'=>'GH','name'=>'Ghana'),
			array('iso'=>'GI','name'=>'Gibraltar'),
			array('iso'=>'GR','name'=>'Greece'),
			array('iso'=>'GL','name'=>'Greenland'),
			array('iso'=>'GD','name'=>'Grenada'),
			array('iso'=>'GP','name'=>'Guadeloupe'),
			array('iso'=>'GU','name'=>'Guam'),
			array('iso'=>'GT','name'=>'Guatemala'),
			array('iso'=>'GN','name'=>'Guinea'),
			array('iso'=>'GW','name'=>'Guinea-Bissau'),
			array('iso'=>'GY','name'=>'Guyana'),
			array('iso'=>'HT','name'=>'Haiti'),
			array('iso'=>'HM','name'=>'Heard And Mc Donald Islands'),
			array('iso'=>'VA','name'=>'Holy See (Vatican City State)'),
			array('iso'=>'HN','name'=>'Honduras'),
			array('iso'=>'HK','name'=>'Hong Kong'),
			array('iso'=>'HU','name'=>'Hungary'),
			array('iso'=>'IS','name'=>'Iceland'),
			array('iso'=>'IN','name'=>'India'),
			array('iso'=>'ID','name'=>'Indonesia'),
			array('iso'=>'IR','name'=>'Iran (Islamic Republic Of)'),
			array('iso'=>'IQ','name'=>'Iraq'),
			array('iso'=>'IE','name'=>'Ireland'),
			array('iso'=>'IL','name'=>'Israel'),
			array('iso'=>'IT','name'=>'Italy'),
			array('iso'=>'JM','name'=>'Jamaica'),
			array('iso'=>'JP','name'=>'Japan'),
			array('iso'=>'JO','name'=>'Jordan'),
			array('iso'=>'KZ','name'=>'Kazakhstan'),
			array('iso'=>'KE','name'=>'Kenya'),
			array('iso'=>'KI','name'=>'Kiribati'),
			array('iso'=>'KP','name'=>'Korea, Democratic People\'s Republic Of'),
			array('iso'=>'KR','name'=>'Korea, Republic Of'),
			array('iso'=>'KW','name'=>'Kuwait'),
			array('iso'=>'KG','name'=>'Kyrgyzstan'),
			array('iso'=>'LA','name'=>'Lao People\'s Democratic Republic'),
			array('iso'=>'LV','name'=>'Latvia'),
			array('iso'=>'LB','name'=>'Lebanon'),
			array('iso'=>'LS','name'=>'Lesotho'),
			array('iso'=>'LR','name'=>'Liberia'),
			array('iso'=>'LY','name'=>'Libyan Arab Jamahiriya'),
			array('iso'=>'LI','name'=>'Liechtenstein'),
			array('iso'=>'LT','name'=>'Lithuania'),
			array('iso'=>'LU','name'=>'Luxembourg'),
			array('iso'=>'MO','name'=>'Macau'),
			array('iso'=>'MK','name'=>'Macedonia, Former Yugoslav Republic Of'),
			array('iso'=>'MG','name'=>'Madagascar'),
			array('iso'=>'MW','name'=>'Malawi'),
			array('iso'=>'MY','name'=>'Malaysia'),
			array('iso'=>'MV','name'=>'Maldives'),
			array('iso'=>'ML','name'=>'Mali'),
			array('iso'=>'MT','name'=>'Malta'),
			array('iso'=>'MH','name'=>'Marshall Islands'),
			array('iso'=>'MQ','name'=>'Martinique'),
			array('iso'=>'MR','name'=>'Mauritania'),
			array('iso'=>'MU','name'=>'Mauritius'),
			array('iso'=>'YT','name'=>'Mayotte'),
			array('iso'=>'MX','name'=>'Mexico'),
			array('iso'=>'FM','name'=>'Micronesia, Federated States Of'),
			array('iso'=>'MD','name'=>'Moldova, Republic Of'),
			array('iso'=>'MC','name'=>'Monaco'),
			array('iso'=>'MN','name'=>'Mongolia'),
			array('iso'=>'MS','name'=>'Montserrat'),
			array('iso'=>'MA','name'=>'Morocco'),
			array('iso'=>'MZ','name'=>'Mozambique'),
			array('iso'=>'MM','name'=>'Myanmar'),
			array('iso'=>'NA','name'=>'Namibia'),
			array('iso'=>'NR','name'=>'Nauru'),
			array('iso'=>'NP','name'=>'Nepal'),
			array('iso'=>'NL','name'=>'Netherlands'),
			array('iso'=>'AN','name'=>'Netherlands Antilles'),
			array('iso'=>'NC','name'=>'New Caledonia'),
			array('iso'=>'NZ','name'=>'New Zealand'),
			array('iso'=>'NI','name'=>'Nicaragua'),
			array('iso'=>'NE','name'=>'Niger'),
			array('iso'=>'NG','name'=>'Nigeria'),
			array('iso'=>'NU','name'=>'Niue'),
			array('iso'=>'NF','name'=>'Norfolk Island'),
			array('iso'=>'MP','name'=>'Northern Mariana Islands'),
			array('iso'=>'NO','name'=>'Norway'),
			array('iso'=>'OM','name'=>'Oman'),
			array('iso'=>'PK','name'=>'Pakistan'),
			array('iso'=>'PW','name'=>'Palau'),
			array('iso'=>'PA','name'=>'Panama'),
			array('iso'=>'PG','name'=>'Papua New Guinea'),
			array('iso'=>'PY','name'=>'Paraguay'),
			array('iso'=>'PE','name'=>'Peru'),
			array('iso'=>'PH','name'=>'Philippines'),
			array('iso'=>'PN','name'=>'Pitcairn'),
			array('iso'=>'PL','name'=>'Poland'),
			array('iso'=>'PT','name'=>'Portugal'),
			array('iso'=>'PR','name'=>'Puerto Rico'),
			array('iso'=>'QA','name'=>'Qatar'),
			array('iso'=>'RE','name'=>'Reunion'),
			array('iso'=>'RO','name'=>'Romania'),
			array('iso'=>'RU','name'=>'Russian Federation'),
			array('iso'=>'RW','name'=>'Rwanda'),
			array('iso'=>'KN','name'=>'Saint Kitts And Nevis'),
			array('iso'=>'LC','name'=>'Saint Lucia'),
			array('iso'=>'VC','name'=>'Saint Vincent And The Grenadines'),
			array('iso'=>'WS','name'=>'Samoa'),
			array('iso'=>'SM','name'=>'San Marino'),
			array('iso'=>'ST','name'=>'Sao Tome And Principe'),
			array('iso'=>'SA','name'=>'Saudi Arabia'),
			array('iso'=>'SN','name'=>'Senegal'),
			array('iso'=>'SC','name'=>'Seychelles'),
			array('iso'=>'SL','name'=>'Sierra Leone'),
			array('iso'=>'SG','name'=>'Singapore'),
			array('iso'=>'SK','name'=>'Slovakia (Slovak Republic)'),
			array('iso'=>'SI','name'=>'Slovenia'),
			array('iso'=>'SB','name'=>'Solomon Islands'),
			array('iso'=>'SO','name'=>'Somalia'),
			array('iso'=>'ZA','name'=>'South Africa'),
			array('iso'=>'GS','name'=>'South Georgia, South Sandwich Islands'),
			array('iso'=>'ES','name'=>'Spain'),
			array('iso'=>'LK','name'=>'Sri Lanka'),
			array('iso'=>'SH','name'=>'St. Helena'),
			array('iso'=>'PM','name'=>'St. Pierre And Miquelon'),
			array('iso'=>'SD','name'=>'Sudan'),
			array('iso'=>'SR','name'=>'Suriname'),
			array('iso'=>'SJ','name'=>'Svalbard And Jan Mayen Islands'),
			array('iso'=>'SZ','name'=>'Swaziland'),
			array('iso'=>'SE','name'=>'Sweden'),
			array('iso'=>'CH','name'=>'Switzerland'),
			array('iso'=>'SY','name'=>'Syrian Arab Republic'),
			array('iso'=>'TW','name'=>'Taiwan'),
			array('iso'=>'TJ','name'=>'Tajikistan'),
			array('iso'=>'TZ','name'=>'Tanzania, United Republic Of'),
			array('iso'=>'TH','name'=>'Thailand'),
			array('iso'=>'TG','name'=>'Togo'),
			array('iso'=>'TK','name'=>'Tokelau'),
			array('iso'=>'TO','name'=>'Tonga'),
			array('iso'=>'TT','name'=>'Trinidad And Tobago'),
			array('iso'=>'TN','name'=>'Tunisia'),
			array('iso'=>'TR','name'=>'Turkey'),
			array('iso'=>'TM','name'=>'Turkmenistan'),
			array('iso'=>'TC','name'=>'Turks And Caicos Islands'),
			array('iso'=>'TV','name'=>'Tuvalu'),
			array('iso'=>'UG','name'=>'Uganda'),
			array('iso'=>'UA','name'=>'Ukraine'),
			array('iso'=>'AE','name'=>'United Arab Emirates'),
			array('iso'=>'UM','name'=>'United States Minor Outlying Islands'),
			array('iso'=>'UY','name'=>'Uruguay'),
			array('iso'=>'UZ','name'=>'Uzbekistan'),
			array('iso'=>'VU','name'=>'Vanuatu'),
			array('iso'=>'VE','name'=>'Venezuela'),
			array('iso'=>'VN','name'=>'Viet Nam'),
			array('iso'=>'VG','name'=>'Virgin Islands (British)'),
			array('iso'=>'VI','name'=>'Virgin Islands (U.S.)'),
			array('iso'=>'WF','name'=>'Wallis And Futuna Islands'),
			array('iso'=>'EH','name'=>'Western Sahara'),
			array('iso'=>'YE','name'=>'Yemen'),
			array('iso'=>'YU','name'=>'Yugoslavia'),
			array('iso'=>'ZM','name'=>'Zambia'),
			array('iso'=>'ZW','name'=>'Zimbabwe')
	    );

	    foreach($items as $key=>$val){
	    	if($val['iso']==$selected){$d=' selected="selected"';}else{$d='';}
			$options[]=array('selected'=>''.$d.'','value'=>''.$val['name'].'','iso'=>''.$val['iso'].'');
	    }

	    return $options;
	}

	/*********************************************************
		[which way to sort type and associated labels]
	*********************************************************/
	private function ilw_val_sort($sort,$exclude,$values=false){

		$allTypes=$this->ilw_display_options();
		$excluded=explode(",",$exclude);
		$sortOrder=explode(",",$sort);
		if($values){
			$values=explode(",",$values);
		}

		$order=array();
		foreach($allTypes as $k=>$val){
			if(!in_array($val['id'],$excluded)){
				/**if we want to return values with the sortorder**/
				if($values){
					$order[$values[$k]]=''.$sortOrder[$k].'';
				}else{
					$order[$val['id']]=''.$sortOrder[$k].'';
				}

			}
		}
		asort($order);
		return implode(",",array_keys($order));
	}

    private function ilw_affiliate_options($defaults=false) {
		/***default affiliate values***/
		$items['default']['inputtype']='hidden';//set to hidden/text to show/hide
		$items['default']['txt']='<b>'.__('I do not have any affiliate relationships and/or would like to contribute to the development of this plugin.', $this->pluginLocale).'</b><br/>'.__('(this will insert the developers affiliate links if available)', $this->pluginLocale).'';

		$items['default']['territories']['EU']['txt']='Tradedoubler';
		$items['default']['territories']['EU']['programVars'][0]['txt']='Program ID';
		$items['default']['territories']['EU']['programVars'][0]['key']='p';
		$items['default']['territories']['EU']['programVars'][0]['value']='23708';
		$items['default']['territories']['EU']['programVars'][1]['txt']='Website ID';
		$items['default']['territories']['EU']['programVars'][1]['key']='a';
		$items['default']['territories']['EU']['programVars'][1]['value']='1285982';

		$items['default']['territories']['USA']['txt']='LinkShare USA';
		$items['default']['territories']['USA']['programVars'][0]['txt']='Affiliate ID';
		$items['default']['territories']['USA']['programVars'][0]['key']='id';
		$items['default']['territories']['USA']['programVars'][0]['value']='TKovL/QEDv4';

		$items['default']['territories']['CAD']['txt']='LinkShare Canada';
		$items['default']['territories']['CAD']['programVars'][0]['txt']='Affiliate ID';
		$items['default']['territories']['CAD']['programVars'][0]['key']='id';
		$items['default']['territories']['CAD']['programVars'][0]['value']='TKovL/QEDv4';

		$items['default']['territories']['AUSNZ']['txt']='DGM';
		$items['default']['territories']['AUSNZ']['programVars'][0]['txt']='Partner ID';
		$items['default']['territories']['AUSNZ']['programVars'][0]['key']='partnerId';
		$items['default']['territories']['AUSNZ']['programVars'][0]['value']='';
		/*****************************************************************************************************************************
		to add more affiliate territories:
			if we need a new territory, let's say Brasil for example we could add it as $items['default']['territories'] shown below
			it must also be added to the ['custom'] array below in the same way with or without default values
			if more than one affiliate variable add as in the ['EU'] territory
			to create the actual link, edit/add values in function ilwformatLinks in the jsfile as appropriate
			such as :
			1): add tCountry['BR']={'territory':'BRA'};(country iso from ilw_country_list below and 'BRA' as territory set in key)
			(if more than one country , have a look at how the EU object is constructed in the js file. )
			2): also add:
				if(territory=='BRA'){
					link+='[your link construct]';
				}
			---------------------------------------------------------------------
			$items['default']['territories']['BRA']['txt']='this if for brasil';
			$items['default']['territories']['BRA']['programVars'][0]['txt']='enter partner id';//explain what to input
			$items['default']['territories']['BRA']['programVars'][0]['key']='partnerId';//key of unique affiliate variable
			$items['default']['territories']['BRA']['programVars'][0]['value']='';//default value of variable
			[***if more than one variable****]
			$items['default']['territories']['BRA']['programVars'][1]['txt']='enter anotherVar here';//explain what to input
			$items['default']['territories']['BRA']['programVars'][1]['key']='anotherVar';//key of another unique affiliate variable
			$items['default']['territories']['BRA']['programVars'][1]['value']='';//default value of variable
			[and so forth....]
		****************************************************************************************************************************/


    	/***custom affiliate values***/
		$items['custom']['inputtype']='text';
		$items['custom']['txt']='<b>'.__('I want to use my own affiliate program links.', $this->pluginLocale).'</b><br/>'.__('(please enter details as required, leave blank if you have no affiliate agreement in a given territory)', $this->pluginLocale).'';

		$items['custom']['territories']['EU']['txt']='Europe (via Tradedoubler)';
		$items['custom']['territories']['EU']['programVars'][0]['txt']='Program ID';
		$items['custom']['territories']['EU']['programVars'][0]['key']='p';
		$items['custom']['territories']['EU']['programVars'][0]['value']='';
		$items['custom']['territories']['EU']['programVars'][1]['txt']='Website ID';
		$items['custom']['territories']['EU']['programVars'][1]['key']='a';
		$items['custom']['territories']['EU']['programVars'][1]['value']='';

		$items['custom']['territories']['USA']['txt']='USA (via LinkShare)';
		$items['custom']['territories']['USA']['programVars'][0]['txt']='Affiliate ID';
		$items['custom']['territories']['USA']['programVars'][0]['key']='id';
		$items['custom']['territories']['USA']['programVars'][0]['value']='';

		$items['custom']['territories']['CAD']['txt']='Canada (via LinkShare)';
		$items['custom']['territories']['CAD']['programVars'][0]['txt']='Affiliate ID';
		$items['custom']['territories']['CAD']['programVars'][0]['key']='id';
		$items['custom']['territories']['CAD']['programVars'][0]['value']='';

		$items['custom']['territories']['AUSNZ']['txt']='Australia, New Zealand (via DGM)';
		$items['custom']['territories']['AUSNZ']['programVars'][0]['txt']='Partner ID';
		$items['custom']['territories']['AUSNZ']['programVars'][0]['key']='partnerId';
		$items['custom']['territories']['AUSNZ']['programVars'][0]['value']='';

		/***use itunes links***/
		$items['iTunes']['txt']='<b>'.__('I do not want to use any *affilate* links at all. All links go directly to the iTunes store', $this->pluginLocale).'</b>';
		/** dont use any links**/
		$items['none']['txt']='<b>'.__('I do not want to use *any* links at all. Display widget without links to iTunes', $this->pluginLocale).'</b>';

		if($defaults && is_array($defaults)){
			$defaultVars=array();
			foreach($defaults as $k=>$v){
				foreach($items[$v]['territories'] as $l=>$m){
					$defaultVars[$v][$l]=array();
					foreach($m['programVars'] as $x=>$y){
						$defaultVars[$v][$l][$y['key']]=$y['value'];
					}
				}
			}
			return $defaultVars;
		}else{
			return $items;
		}
    }
	/*********************************************************
		[language options]
	*********************************************************/
	private function ilw_language_options($selected=''){
		$items['en_us']=__('English', $this->pluginLocale);
		$items['ja_jp']=__('Japanese', $this->pluginLocale);

	    foreach($items as $key=>$val){
	    	if($key==$selected){$d=' selected="selected"';}else{$d='';}
			$options[]=array('selected'=>''.$d.'','value'=>''.$val.'','id'=>''.$key.'');
	    }
	    return $options;
	}
	/*********************************************************
		[list type sort options]
	*********************************************************/
	private function ilw_list_type_sort_options($selected=''){
		$items['default']=__('Default', $this->pluginLocale);
		$items['itemId']=__('ID', $this->pluginLocale);
		$items['itemName']=__('Name', $this->pluginLocale);
		$items['releaseDate']=__('Release Date', $this->pluginLocale);

	    foreach($items as $key=>$val){
	    	if($key==$selected){$d=' selected="selected"';}else{$d='';}
			$options[]=array('selected'=>''.$d.'','value'=>''.$val.'','id'=>''.$key.'');
	    }

	    return $options;
	}
	/*********************************************************
		[list type sort order]
	*********************************************************/
	private function ilw_list_type_sort_order($selected=''){
		$items['asc']='&uArr;';
		$items['desc']='&dArr;';

	    foreach($items as $key=>$val){
	    	if($key==$selected){$d=' selected="selected"';}else{$d='';}
			$options[]=array('selected'=>''.$d.'','value'=>''.$val.'','id'=>''.$key.'');
	    }

	    return $options;
	}
	/*********************************************************
		[which items to display options]
	*********************************************************/
	private function ilw_display_options($selected='',$singleDimension=false){
		$items['collection']=__('Collections/Albums', $this->pluginLocale);
		$items['featureMovie']=__('Feature Movies', $this->pluginLocale);
		$items['podcast']=__('Podcasts', $this->pluginLocale);
		$items['audiobook']=__('Audiobooks', $this->pluginLocale);
		$items['ebook']=__('eBooks', $this->pluginLocale);
		$items['software']=__('Software', $this->pluginLocale);

		$s=explode(',',$selected);
	    foreach($items as $key=>$val){
	    	if(in_array($key,$s)){$d=' checked="checked"';}else{$d='';}
			$options[]=array('selected'=>''.$d.'','value'=>''.$val.'','id'=>''.$key.'');
	    }
	    if($singleDimension){
	    	$options=$items;
	    }

	    return $options;
	}
	/*********************************************************
		[localize frontend options]
	*********************************************************/
	private function ilw_localize_frontend($selVal='val'){
		$items['Released']=array('label'=>__('Label for iTunes Release Date (if present)', $this->pluginLocale),'val'=>__('iTunes Release: ', $this->pluginLocale));
		$items['Genre']=array('label'=>__('Label for Genre (if present)', $this->pluginLocale),'val'=>__('Genre: ', $this->pluginLocale));
		$items['UserRating']=array('label'=>__('Label for Average User Rating (if present)', $this->pluginLocale),'val'=>__('Average User Rating: ', $this->pluginLocale));
		$items['iTunes']=array('label'=>__('Linktext to iTunes (Note: if left blank, there won\'t be any link !!)', $this->pluginLocale),'val'=>__('iTunes', $this->pluginLocale));
		$items['viewTracklist']=array('label'=>__('Linktext to view Tracklist', $this->pluginLocale),'val'=>__('view Tracklist', $this->pluginLocale));
		$items['viewDescription']=array('label'=>__('Linktext to view full description (if present)', $this->pluginLocale),'val'=>__('full Description', $this->pluginLocale));
		$items['Back']=array('label'=>__('Linktext to go back when viewing details (such as tracklists)', $this->pluginLocale),'val'=>__('back', $this->pluginLocale));
		$items['Close']=array('label'=>__('Linktext to close details (such as full descriptions)', $this->pluginLocale),'val'=>__('[close]', $this->pluginLocale));
		$items['Free']=array('label'=>__('if iTunes price is free / 0.00', $this->pluginLocale),'val'=>__('free', $this->pluginLocale));

	    foreach($items as $k=>$v){
			$options[$k]=$v[$selVal];
	    }

	return $options;
	}
	/*********************************************************
		[explicit options]
	*********************************************************/
	private function ilw_explicit_options($selected=''){
		$items['Yes']=__('Yes', $this->pluginLocale);
		$items['No']=__('No', $this->pluginLocale);

	    foreach($items as $key=>$val){
	    	if($key==$selected){$d=' selected="selected"';}else{$d='';}
			$options[]=array('selected'=>''.$d.'','value'=>''.$val.'','id'=>''.$key.'');
	    }

	    return $options;
	}

} // end class
add_action('widgets_init', create_function('', 'register_widget("iTunes_Lookup_Widget");'));
?>